@@ -8,7 +8,7 @@ Copyright:
     Copyright(C) 1996-2001 Takuya OOURA
     email: ooura@mmm.t.u-tokyo.ac.jp
     download: http://momonga.t.u-tokyo.ac.jp/~ooura/fft.html
-    You may use, copy, modify this code for any purpose and 
+    You may use, copy, modify this code for any purpose and
     without fee. You may distribute this ORIGINAL package.
 
 After the following prologue, the code is essentially Ooura's original, which I
@@ -21,13 +21,13 @@ for another, more permissive-sounding copyright notice.  -MSP
 /* ---------- Pd interface to OOURA FFT; imitate Mayer API ---------- */
 #include "m_pd.h"
 
-#ifdef HAVE_ALLOCA_H        /* ifdef nonsense to find include for alloca() */
-# include <alloca.h>        /* linux, mac, mingw, cygwin */
-#elif defined _MSC_VER
-# include <malloc.h>        /* MSVC */
+#ifdef _WIN32
+# include <malloc.h> /* MSVC or mingw on windows */
+#elif defined(__linux__) || defined(__APPLE__)
+# include <alloca.h> /* linux, mac, mingw, cygwin */
 #else
-# include <stddef.h>        /* BSDs for example */
-#endif                      /* end alloca() ifdef nonsense */
+# include <stdlib.h> /* BSDs for example */
+#endif
 
 #define FFTFLT double
 void cdft(int, int, FFTFLT *, int *, FFTFLT *);
@@ -216,23 +216,23 @@ macro definitions
                         n >= 1, n = power of 2
         a[0...2*n-1]   :input/output data (FFTFLT *)
                         input data
-                            a[2*j] = Re(x[j]), 
+                            a[2*j] = Re(x[j]),
                             a[2*j+1] = Im(x[j]), 0<=j<n
                         output data
-                            a[2*k] = Re(X[k]), 
+                            a[2*k] = Re(X[k]),
                             a[2*k+1] = Im(X[k]), 0<=k<n
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n/2-1]   :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             cdft(2*n, -1, a, ip, w);
-        is 
+        is
             cdft(2*n, 1, a, ip, w);
             for (j = 0; j <= 2 * n - 1; j++) {
                 a[j] *= 1.0 / n;
@@ -246,8 +246,8 @@ macro definitions
             R[k] = sum_j=0^n-1 a[j]*cos(2*pi*j*k/n), 0<=k<=n/2
             I[k] = sum_j=0^n-1 a[j]*sin(2*pi*j*k/n), 0<k<n/2
         <case2> IRDFT (excluding scale)
-            a[k] = (R[0] + R[n/2]*cos(pi*k))/2 + 
-                   sum_j=1^n/2-1 R[j]*cos(2*pi*j*k/n) + 
+            a[k] = (R[0] + R[n/2]*cos(pi*k))/2 +
+                   sum_j=1^n/2-1 R[j]*cos(2*pi*j*k/n) +
                    sum_j=1^n/2-1 I[j]*sin(2*pi*j*k/n), 0<=k<n
     [usage]
         <case1>
@@ -272,16 +272,16 @@ macro definitions
                                 a[1] = R[n/2]
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n/2)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n/2+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n/2-1]   :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             rdft(n, 1, a, ip, w);
-        is 
+        is
             rdft(n, -1, a, ip, w);
             for (j = 0; j <= n - 1; j++) {
                 a[j] *= 2.0 / n;
@@ -310,16 +310,16 @@ macro definitions
                             a[k] = C[k], 0<=k<n
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n/2)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n/2+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n*5/4-1] :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             ddct(n, -1, a, ip, w);
-        is 
+        is
             a[0] *= 0.5;
             ddct(n, 1, a, ip, w);
             for (j = 0; j <= n - 1; j++) {
@@ -357,16 +357,16 @@ macro definitions
                                 a[0] = S[n]
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n/2)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n/2+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n*5/4-1] :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             ddst(n, -1, a, ip, w);
-        is 
+        is
             a[0] *= 0.5;
             ddst(n, 1, a, ip, w);
             for (j = 0; j <= n - 1; j++) {
@@ -390,18 +390,18 @@ macro definitions
         t[0...n/2]     :work area (FFTFLT *)
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n/4)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n/4+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n*5/8-1] :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             a[0] *= 0.5;
             a[n] *= 0.5;
             dfct(n, a, t, ip, w);
-        is 
+        is
             a[0] *= 0.5;
             a[n] *= 0.5;
             dfct(n, a, t, ip, w);
@@ -427,16 +427,16 @@ macro definitions
         t[0...n/2-1]   :work area (FFTFLT *)
         ip[0...*]      :work area for bit reversal (int *)
                         length of ip >= 2+sqrt(n/4)
-                        strictly, 
-                        length of ip >= 
+                        strictly,
+                        length of ip >=
                             2+(1<<(int)(log(n/4+0.5)/log(2))/2).
                         ip[0],ip[1] are pointers of the cos/sin table.
         w[0...n*5/8-1] :cos/sin table (FFTFLT *)
                         w[],ip[] are initialized if ip[0] == 0.
     [remark]
-        Inverse of 
+        Inverse of
             dfst(n, a, t, ip, w);
-        is 
+        is
             dfst(n, a, t, ip, w);
             for (j = 1; j <= n - 1; j++) {
                 a[j] *= 2.0 / n;
@@ -456,7 +456,7 @@ void cdft(int n, int isgn, FFTFLT *a, int *ip, FFTFLT *w)
     void cftfsub(int n, FFTFLT *a, int *ip, int nw, FFTFLT *w);
     void cftbsub(int n, FFTFLT *a, int *ip, int nw, FFTFLT *w);
     int nw;
-    
+
     nw = ip[0];
     if (n > (nw << 2)) {
         nw = n >> 2;
@@ -480,7 +480,7 @@ void rdft(int n, int isgn, FFTFLT *a, int *ip, FFTFLT *w)
     void rftbsub(int n, FFTFLT *a, int nc, FFTFLT *c);
     int nw, nc;
     FFTFLT xi;
-    
+
     nw = ip[0];
     if (n > (nw << 2)) {
         nw = n >> 2;
@@ -525,7 +525,7 @@ void ddct(int n, int isgn, FFTFLT *a, int *ip, FFTFLT *w)
     void dctsub(int n, FFTFLT *a, int nc, FFTFLT *c);
     int j, nw, nc;
     FFTFLT xr;
-    
+
     nw = ip[0];
     if (n > (nw << 2)) {
         nw = n >> 2;
@@ -581,7 +581,7 @@ void ddst(int n, int isgn, FFTFLT *a, int *ip, FFTFLT *w)
     void dstsub(int n, FFTFLT *a, int nc, FFTFLT *c);
     int j, nw, nc;
     FFTFLT xr;
-    
+
     nw = ip[0];
     if (n > (nw << 2)) {
         nw = n >> 2;
@@ -635,7 +635,7 @@ void dfct(int n, FFTFLT *a, FFTFLT *t, int *ip, FFTFLT *w)
     void dctsub(int n, FFTFLT *a, int nc, FFTFLT *c);
     int j, k, l, m, mh, nw, nc;
     FFTFLT xr, xi, yr, yi;
-    
+
     nw = ip[0];
     if (n > (nw << 3)) {
         nw = n >> 3;
@@ -728,7 +728,7 @@ void dfst(int n, FFTFLT *a, FFTFLT *t, int *ip, FFTFLT *w)
     void dstsub(int n, FFTFLT *a, int nc, FFTFLT *c);
     int j, k, l, m, mh, nw, nc;
     FFTFLT xr, xi, yr, yi;
-    
+
     nw = ip[0];
     if (n > (nw << 3)) {
         nw = n >> 3;
@@ -813,7 +813,7 @@ void makewt(int nw, int *ip, FFTFLT *w)
     void makeipt(int nw, int *ip);
     int j, nwh, nw0, nw1;
     FFTFLT delta, wn4r, wk1r, wk1i, wk3r, wk3i;
-    
+
     ip[0] = nw;
     ip[1] = 1;
     if (nw > 2) {
@@ -872,7 +872,7 @@ void makewt(int nw, int *ip, FFTFLT *w)
 void makeipt(int nw, int *ip)
 {
     int j, l, m, m2, p, q;
-    
+
     ip[2] = 0;
     ip[3] = 16;
     m = 2;
@@ -893,7 +893,7 @@ void makect(int nc, int *ip, FFTFLT *c)
 {
     int j, nch;
     FFTFLT delta;
-    
+
     ip[1] = nc;
     if (nc > 1) {
         nch = nc >> 1;
@@ -981,14 +981,14 @@ void cftfsub(int n, FFTFLT *a, int *ip, int nw, FFTFLT *w)
 #ifdef USE_CDFT_THREADS
     void cftrec4_th(int n, FFTFLT *a, int nw, FFTFLT *w);
 #endif /* USE_CDFT_THREADS */
-    
+
     if (n > 8) {
         if (n > 32) {
             cftf1st(n, a, &w[nw - (n >> 2)]);
 #ifdef USE_CDFT_THREADS
             if (n > CDFT_THREADS_BEGIN_N) {
                 cftrec4_th(n, a, nw, w);
-            } else 
+            } else
 #endif /* USE_CDFT_THREADS */
             if (n > 512) {
                 cftrec4(n, a, nw, w);
@@ -1029,14 +1029,14 @@ void cftbsub(int n, FFTFLT *a, int *ip, int nw, FFTFLT *w)
 #ifdef USE_CDFT_THREADS
     void cftrec4_th(int n, FFTFLT *a, int nw, FFTFLT *w);
 #endif /* USE_CDFT_THREADS */
-    
+
     if (n > 8) {
         if (n > 32) {
             cftb1st(n, a, &w[nw - (n >> 2)]);
 #ifdef USE_CDFT_THREADS
             if (n > CDFT_THREADS_BEGIN_N) {
                 cftrec4_th(n, a, nw, w);
-            } else 
+            } else
 #endif /* USE_CDFT_THREADS */
             if (n > 512) {
                 cftrec4(n, a, nw, w);
@@ -1065,7 +1065,7 @@ void bitrv2(int n, int *ip, FFTFLT *a)
 {
     int j, j1, k, k1, l, m, nh, nm;
     FFTFLT xr, xi, yr, yi;
-    
+
     m = 1;
     for (l = n >> 2; l > 8; l >>= 2) {
         m <<= 1;
@@ -1412,7 +1412,7 @@ void bitrv2conj(int n, int *ip, FFTFLT *a)
 {
     int j, j1, k, k1, l, m, nh, nm;
     FFTFLT xr, xi, yr, yi;
-    
+
     m = 1;
     for (l = n >> 2; l > 8; l >>= 2) {
         m <<= 1;
@@ -1765,10 +1765,10 @@ void bitrv2conj(int n, int *ip, FFTFLT *a)
 
 void bitrv216(FFTFLT *a)
 {
-    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i, 
-        x5r, x5i, x7r, x7i, x8r, x8i, x10r, x10i, 
+    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i,
+        x5r, x5i, x7r, x7i, x8r, x8i, x10r, x10i,
         x11r, x11i, x12r, x12i, x13r, x13i, x14r, x14i;
-    
+
     x1r = a[2];
     x1i = a[3];
     x2r = a[4];
@@ -1822,11 +1822,11 @@ void bitrv216(FFTFLT *a)
 
 void bitrv216neg(FFTFLT *a)
 {
-    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i, 
-        x5r, x5i, x6r, x6i, x7r, x7i, x8r, x8i, 
-        x9r, x9i, x10r, x10i, x11r, x11i, x12r, x12i, 
+    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i,
+        x5r, x5i, x6r, x6i, x7r, x7i, x8r, x8i,
+        x9r, x9i, x10r, x10i, x11r, x11i, x12r, x12i,
         x13r, x13i, x14r, x14i, x15r, x15i;
-    
+
     x1r = a[2];
     x1i = a[3];
     x2r = a[4];
@@ -1893,7 +1893,7 @@ void bitrv216neg(FFTFLT *a)
 void bitrv208(FFTFLT *a)
 {
     FFTFLT x1r, x1i, x3r, x3i, x4r, x4i, x6r, x6i;
-    
+
     x1r = a[2];
     x1i = a[3];
     x3r = a[6];
@@ -1915,9 +1915,9 @@ void bitrv208(FFTFLT *a)
 
 void bitrv208neg(FFTFLT *a)
 {
-    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i, 
+    FFTFLT x1r, x1i, x2r, x2i, x3r, x3i, x4r, x4i,
         x5r, x5i, x6r, x6i, x7r, x7i;
-    
+
     x1r = a[2];
     x1i = a[3];
     x2r = a[4];
@@ -1952,11 +1952,11 @@ void bitrv208neg(FFTFLT *a)
 void cftf1st(int n, FFTFLT *a, FFTFLT *w)
 {
     int j, j0, j1, j2, j3, k, m, mh;
-    FFTFLT wn4r, csc1, csc3, wk1r, wk1i, wk3r, wk3i, 
+    FFTFLT wn4r, csc1, csc3, wk1r, wk1i, wk3r, wk3i,
         wd1r, wd1i, wd3r, wd3i;
-    FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i, 
+    FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i,
         y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i;
-    
+
     mh = n >> 3;
     m = 2 * mh;
     j1 = m;
@@ -2158,11 +2158,11 @@ void cftf1st(int n, FFTFLT *a, FFTFLT *w)
 void cftb1st(int n, FFTFLT *a, FFTFLT *w)
 {
     int j, j0, j1, j2, j3, k, m, mh;
-    FFTFLT wn4r, csc1, csc3, wk1r, wk1i, wk3r, wk3i, 
+    FFTFLT wn4r, csc1, csc3, wk1r, wk1i, wk3r, wk3i,
         wd1r, wd1i, wd3r, wd3i;
-    FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i, 
+    FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i,
         y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i;
-    
+
     mh = n >> 3;
     m = 2 * mh;
     j1 = m;
@@ -2379,7 +2379,7 @@ void cftrec4_th(int n, FFTFLT *a, int nw, FFTFLT *w)
     int i, idiv4, m, nthread;
     cdft_thread_t th[4];
     cdft_arg_t ag[4];
-    
+
     nthread = 2;
     idiv4 = 0;
     m = n >> 1;
@@ -2413,7 +2413,7 @@ void *cftrec1_th(void *p)
     void cftmdl1(int n, FFTFLT *a, FFTFLT *w);
     int isplt, j, k, m, n, n0, nw;
     FFTFLT *a, *w;
-    
+
     n0 = ((cdft_arg_t *) p)->n0;
     n = ((cdft_arg_t *) p)->n;
     a = ((cdft_arg_t *) p)->a;
@@ -2442,7 +2442,7 @@ void *cftrec2_th(void *p)
     void cftmdl2(int n, FFTFLT *a, FFTFLT *w);
     int isplt, j, k, m, n, n0, nw;
     FFTFLT *a, *w;
-    
+
     n0 = ((cdft_arg_t *) p)->n0;
     n = ((cdft_arg_t *) p)->n;
     a = ((cdft_arg_t *) p)->a;
@@ -2473,7 +2473,7 @@ void cftrec4(int n, FFTFLT *a, int nw, FFTFLT *w)
     void cftleaf(int n, int isplt, FFTFLT *a, int nw, FFTFLT *w);
     void cftmdl1(int n, FFTFLT *a, FFTFLT *w);
     int isplt, j, k, m;
-    
+
     m = n;
     while (m > 512) {
         m >>= 2;
@@ -2494,7 +2494,7 @@ int cfttree(int n, int j, int k, FFTFLT *a, int nw, FFTFLT *w)
     void cftmdl1(int n, FFTFLT *a, FFTFLT *w);
     void cftmdl2(int n, FFTFLT *a, FFTFLT *w);
     int i, isplt, m;
-    
+
     if ((k & 3) != 0) {
         isplt = k & 1;
         if (isplt != 0) {
@@ -2532,7 +2532,7 @@ void cftleaf(int n, int isplt, FFTFLT *a, int nw, FFTFLT *w)
     void cftf162(FFTFLT *a, FFTFLT *w);
     void cftf081(FFTFLT *a, FFTFLT *w);
     void cftf082(FFTFLT *a, FFTFLT *w);
-    
+
     if (n == 512) {
         cftmdl1(128, a, &w[nw - 64]);
         cftf161(a, &w[nw - 8]);
@@ -2594,7 +2594,7 @@ void cftmdl1(int n, FFTFLT *a, FFTFLT *w)
     int j, j0, j1, j2, j3, k, m, mh;
     FFTFLT wn4r, wk1r, wk1i, wk3r, wk3i;
     FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
-    
+
     mh = n >> 3;
     m = 2 * mh;
     j1 = m;
@@ -2704,7 +2704,7 @@ void cftmdl2(int n, FFTFLT *a, FFTFLT *w)
     int j, j0, j1, j2, j3, k, kr, m, mh;
     FFTFLT wn4r, wk1r, wk1i, wk3r, wk3i, wd1r, wd1i, wd3r, wd3i;
     FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i, y0r, y0i, y2r, y2i;
-    
+
     mh = n >> 3;
     m = 2 * mh;
     wn4r = w[1];
@@ -2839,7 +2839,7 @@ void cftfx41(int n, FFTFLT *a, int nw, FFTFLT *w)
     void cftf162(FFTFLT *a, FFTFLT *w);
     void cftf081(FFTFLT *a, FFTFLT *w);
     void cftf082(FFTFLT *a, FFTFLT *w);
-    
+
     if (n == 128) {
         cftf161(a, &w[nw - 8]);
         cftf162(&a[32], &w[nw - 32]);
@@ -2856,13 +2856,13 @@ void cftfx41(int n, FFTFLT *a, int nw, FFTFLT *w)
 
 void cftf161(FFTFLT *a, FFTFLT *w)
 {
-    FFTFLT wn4r, wk1r, wk1i, 
-        x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i, 
-        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i, 
-        y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i, 
-        y8r, y8i, y9r, y9i, y10r, y10i, y11r, y11i, 
+    FFTFLT wn4r, wk1r, wk1i,
+        x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i,
+        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i,
+        y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i,
+        y8r, y8i, y9r, y9i, y10r, y10i, y11r, y11i,
         y12r, y12i, y13r, y13i, y14r, y14i, y15r, y15i;
-    
+
     wn4r = w[1];
     wk1r = w[2];
     wk1i = w[3];
@@ -3015,13 +3015,13 @@ void cftf161(FFTFLT *a, FFTFLT *w)
 
 void cftf162(FFTFLT *a, FFTFLT *w)
 {
-    FFTFLT wn4r, wk1r, wk1i, wk2r, wk2i, wk3r, wk3i, 
-        x0r, x0i, x1r, x1i, x2r, x2i, 
-        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i, 
-        y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i, 
-        y8r, y8i, y9r, y9i, y10r, y10i, y11r, y11i, 
+    FFTFLT wn4r, wk1r, wk1i, wk2r, wk2i, wk3r, wk3i,
+        x0r, x0i, x1r, x1i, x2r, x2i,
+        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i,
+        y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i,
+        y8r, y8i, y9r, y9i, y10r, y10i, y11r, y11i,
         y12r, y12i, y13r, y13i, y14r, y14i, y15r, y15i;
-    
+
     wn4r = w[1];
     wk1r = w[4];
     wk1i = w[5];
@@ -3198,10 +3198,10 @@ void cftf162(FFTFLT *a, FFTFLT *w)
 
 void cftf081(FFTFLT *a, FFTFLT *w)
 {
-    FFTFLT wn4r, x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i, 
-        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i, 
+    FFTFLT wn4r, x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i,
+        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i,
         y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i;
-    
+
     wn4r = w[1];
     x0r = a[0] + a[8];
     x0i = a[1] + a[9];
@@ -3260,10 +3260,10 @@ void cftf081(FFTFLT *a, FFTFLT *w)
 
 void cftf082(FFTFLT *a, FFTFLT *w)
 {
-    FFTFLT wn4r, wk1r, wk1i, x0r, x0i, x1r, x1i, 
-        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i, 
+    FFTFLT wn4r, wk1r, wk1i, x0r, x0i, x1r, x1i,
+        y0r, y0i, y1r, y1i, y2r, y2i, y3r, y3i,
         y4r, y4i, y5r, y5i, y6r, y6i, y7r, y7i;
-    
+
     wn4r = w[1];
     wk1r = w[2];
     wk1i = w[3];
@@ -3333,7 +3333,7 @@ void cftf082(FFTFLT *a, FFTFLT *w)
 void cftf040(FFTFLT *a)
 {
     FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
-    
+
     x0r = a[0] + a[4];
     x0i = a[1] + a[5];
     x1r = a[0] - a[4];
@@ -3356,7 +3356,7 @@ void cftf040(FFTFLT *a)
 void cftb040(FFTFLT *a)
 {
     FFTFLT x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
-    
+
     x0r = a[0] + a[4];
     x0i = a[1] + a[5];
     x1r = a[0] - a[4];
@@ -3379,7 +3379,7 @@ void cftb040(FFTFLT *a)
 void cftx020(FFTFLT *a)
 {
     FFTFLT x0r, x0i;
-    
+
     x0r = a[0] - a[2];
     x0i = a[1] - a[3];
     a[0] += a[2];
@@ -3393,7 +3393,7 @@ void rftfsub(int n, FFTFLT *a, int nc, FFTFLT *c)
 {
     int j, k, kk, ks, m;
     FFTFLT wkr, wki, xr, xi, yr, yi;
-    
+
     m = n >> 1;
     ks = 2 * nc / m;
     kk = 0;
@@ -3418,7 +3418,7 @@ void rftbsub(int n, FFTFLT *a, int nc, FFTFLT *c)
 {
     int j, k, kk, ks, m;
     FFTFLT wkr, wki, xr, xi, yr, yi;
-    
+
     m = n >> 1;
     ks = 2 * nc / m;
     kk = 0;
@@ -3443,7 +3443,7 @@ void dctsub(int n, FFTFLT *a, int nc, FFTFLT *c)
 {
     int j, k, kk, ks, m;
     FFTFLT wkr, wki, xr;
-    
+
     m = n >> 1;
     ks = nc / n;
     kk = 0;
@@ -3464,7 +3464,7 @@ void dstsub(int n, FFTFLT *a, int nc, FFTFLT *c)
 {
     int j, k, kk, ks, m;
     FFTFLT wkr, wki, xr;
-    
+
     m = n >> 1;
     ks = nc / n;
     kk = 0;
