@@ -108,7 +108,7 @@ void rtext_getseltext(t_rtext *x, char **buf, int *bufsize)
 /* convert t_text te_type symbol for use as a Tk tag */
 static t_symbol *rtext_gettype(t_rtext *x)
 {
-    switch (x->x_text->te_type) 
+    switch (x->x_text->te_type)
     {
     case T_TEXT: return gensym("text");
     case T_OBJECT: return gensym("obj");
@@ -165,7 +165,7 @@ static int lastone(char *s, int c, int n)
     of the entire text in pixels.
     */
 
-   /*-- moo: 
+   /*-- moo:
     * + some variables from the original version have been renamed
     * + variables with a "_b" suffix are raw byte strings, lengths, or offsets
     * + variables with a "_c" suffix are logical character lengths or offsets
@@ -201,10 +201,17 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     if (pd_class(&x->x_text->te_pd) == canvas_class &&
         ((t_glist *)(x->x_text))->gl_isgraph &&
         ((t_glist *)(x->x_text))->gl_goprect)
-            font =  glist_getfont((t_glist *)(x->x_text));
-    else font = glist_getfont(x->x_glist);
-    fontwidth = sys_fontwidth(font);
-    fontheight = sys_fontheight(font);
+    {
+        font =  glist_getfont((t_glist *)(x->x_text));
+        fontwidth =  glist_fontwidth((t_glist *)(x->x_text));
+        fontheight =  glist_fontheight((t_glist *)(x->x_text));
+    }
+    else
+    {
+        font = glist_getfont(x->x_glist);
+        fontwidth = glist_fontwidth(x->x_glist);
+        fontheight = glist_fontheight(x->x_glist);
+    }
     findx = (*widthp + (fontwidth/2)) / fontwidth;
     findy = *heightp / fontheight;
     if (x->x_bufsize >= 100)
@@ -306,7 +313,8 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
         sys_vgui("pdtk_text_new .x%lx.c {%s %s text} %f %f {%.*s} %d %s\n",
             canvas, x->x_tag, rtext_gettype(x)->s_name,
             dispx + LMARGIN, dispy + TMARGIN,
-            outchars_b, tempbuf, sys_hostfontsize(font),
+            outchars_b, tempbuf,
+            sys_hostfontsize(font, glist_getzoom(x->x_glist)),
             (glist_isselected(x->x_glist,
                 &x->x_glist->gl_gobj)? "blue" : "black"));
     }
@@ -314,31 +322,31 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     {
         sys_vgui("pdtk_text_set .x%lx.c %s {%.*s}\n",
             canvas, x->x_tag, outchars_b, tempbuf);
-        if (pixwide != x->x_drawnwidth || pixhigh != x->x_drawnheight) 
+        if (pixwide != x->x_drawnwidth || pixhigh != x->x_drawnheight)
             text_drawborder(x->x_text, x->x_glist, x->x_tag,
                 pixwide, pixhigh, 0);
         if (x->x_active)
         {
             if (selend_b > selstart_b)
             {
-                sys_vgui(".x%lx.c select from %s %d\n", canvas, 
+                sys_vgui(".x%lx.c select from %s %d\n", canvas,
                     x->x_tag, u8_charnum(x->x_buf, selstart_b));
-                sys_vgui(".x%lx.c select to %s %d\n", canvas, 
+                sys_vgui(".x%lx.c select to %s %d\n", canvas,
                     x->x_tag, u8_charnum(x->x_buf, selend_b) - 1);
-                sys_vgui(".x%lx.c focus \"\"\n", canvas);        
+                sys_vgui(".x%lx.c focus \"\"\n", canvas);
             }
             else
             {
                 sys_vgui(".x%lx.c select clear\n", canvas);
                 sys_vgui(".x%lx.c icursor %s %d\n", canvas, x->x_tag,
                     u8_charnum(x->x_buf, selstart_b));
-                sys_vgui(".x%lx.c focus %s\n", canvas, x->x_tag);        
+                sys_vgui(".x%lx.c focus %s\n", canvas, x->x_tag);
             }
         }
     }
     x->x_drawnwidth = pixwide;
     x->x_drawnheight = pixhigh;
-    
+
     *widthp = pixwide;
     *heightp = pixhigh;
     if (tempbuf != smallbuf)
@@ -406,9 +414,14 @@ t_rtext *glist_findrtext(t_glist *gl, t_text *who)
     t_rtext *x;
     if (!gl->gl_editor)
         canvas_create_editor(gl);
+    {
+        int i = 0;
+        for (x = gl->gl_editor->e_rtext; x && x->x_text != who; x = x->x_next)
+            i++;
+        // post("i=%d", i);
+    }
     for (x = gl->gl_editor->e_rtext; x && x->x_text != who; x = x->x_next)
         ;
-    if (!x) bug("glist_findrtext");
     return (x);
 }
 
@@ -439,7 +452,7 @@ void rtext_erase(t_rtext *x)
 
 void rtext_displace(t_rtext *x, int dx, int dy)
 {
-    sys_vgui(".x%lx.c move %s %d %d\n", glist_getcanvas(x->x_glist), 
+    sys_vgui(".x%lx.c move %s %d %d\n", glist_getcanvas(x->x_glist),
         x->x_tag, dx, dy);
 }
 
@@ -447,7 +460,7 @@ void rtext_select(t_rtext *x, int state)
 {
     t_glist *glist = x->x_glist;
     t_canvas *canvas = glist_getcanvas(glist);
-    sys_vgui(".x%lx.c itemconfigure %s -fill %s\n", canvas, 
+    sys_vgui(".x%lx.c itemconfigure %s -fill %s\n", canvas,
         x->x_tag, (state? "blue" : "black"));
 }
 
@@ -499,7 +512,7 @@ void rtext_key(t_rtext *x, int keynum, t_symbol *keysym)
             if (x->x_selend < x->x_bufsize && (x->x_selstart == x->x_selend))
                 u8_inc(x->x_buf, &x->x_selend);
         }
-        
+
         ndel = x->x_selend - x->x_selstart;
         for (i = x->x_selend; i < x->x_bufsize; i++)
             x->x_buf[i- ndel] = x->x_buf[i];
@@ -605,7 +618,7 @@ void rtext_mouse(t_rtext *x, int xval, int yval, int flag)
         if ((newseparator = lastone(x->x_buf, ',', indx)) > whereseparator)
             whereseparator = newseparator+1;
         x->x_selstart = whereseparator;
-        
+
         whereseparator = x->x_bufsize - indx;
         if ((newseparator =
             firstone(x->x_buf+indx, ' ', x->x_bufsize - indx)) >= 0 &&
