@@ -69,7 +69,7 @@ void gstub_cutoff(t_gstub *gs)
 
 /* call this to verify that a pointer is fresh, i.e., that it either
 points to real data or to the head of a list, and that in either case
-the object hasn't disappeared since this pointer was generated. 
+the object hasn't disappeared since this pointer was generated.
 Unless "headok" is set,  the routine also fails for the head of a list. */
 
 int gpointer_check(const t_gpointer *gp, int headok)
@@ -126,7 +126,7 @@ void gpointer_copy(const t_gpointer *gpfrom, t_gpointer *gpto)
 void gpointer_unset(t_gpointer *gp)
 {
     t_gstub *gs;
-    if (gs = gp->gp_stub)
+    if ((gs = gp->gp_stub))
     {
         gstub_dis(gs);
         gp->gp_stub = 0;
@@ -136,7 +136,7 @@ void gpointer_unset(t_gpointer *gp)
 void gpointer_setglist(t_gpointer *gp, t_glist *glist, t_scalar *x)
 {
     t_gstub *gs;
-    if (gs = gp->gp_stub) gstub_dis(gs);
+    if ((gs = gp->gp_stub)) gstub_dis(gs);
     gp->gp_stub = gs = glist->gl_stub;
     gp->gp_valid = glist->gl_valid;
     gp->gp_un.gp_scalar = x;
@@ -146,7 +146,7 @@ void gpointer_setglist(t_gpointer *gp, t_glist *glist, t_scalar *x)
 void gpointer_setarray(t_gpointer *gp, t_array *array, t_word *w)
 {
     t_gstub *gs;
-    if (gs = gp->gp_stub) gstub_dis(gs);
+    if ((gs = gp->gp_stub)) gstub_dis(gs);
     gp->gp_stub = gs = array->a_stub;
     gp->gp_valid = array->a_valid;
     gp->gp_un.gp_w = w;
@@ -217,7 +217,7 @@ static t_symbol *template_getbindsym(t_symbol *s)
 
 static t_class *ptrobj_class;
 
-typedef struct 
+typedef struct
 {
     t_symbol *to_type;
     t_outlet *to_outlet;
@@ -290,13 +290,13 @@ static void ptrobj_vnext(t_ptrobj *x, t_float f)
         return;
     }
     gobj = &gp->gp_un.gp_scalar->sc_gobj;
-    
+
     if (!gobj) gobj = glist->gl_list;
     else gobj = gobj->g_next;
     while (gobj && ((pd_class(&gobj->g_pd) != scalar_class) ||
         (wantselected && !glist_isselected(glist, gobj))))
             gobj = gobj->g_next;
-    
+
     if (gobj)
     {
         t_typedout *to;
@@ -304,7 +304,7 @@ static void ptrobj_vnext(t_ptrobj *x, t_float f)
         t_scalar *sc = (t_scalar *)gobj;
         t_symbol *templatesym = sc->sc_template;
 
-        gp->gp_un.gp_scalar = sc; 
+        gp->gp_un.gp_scalar = sc;
         for (n = x->x_ntypedout, to = x->x_typedout; n--; to++)
         {
             if (to->to_type == templatesym)
@@ -344,13 +344,13 @@ static void ptrobj_sendwindow(t_ptrobj *x, t_symbol *s, int argc, t_atom *argv)
     }
     gs = x->x_gp.gp_stub;
     if (gs->gs_which == GP_GLIST)
-        glist = gs->gs_un.gs_glist;  
+        glist = gs->gs_un.gs_glist;
     else
     {
         t_array *owner_array = gs->gs_un.gs_array;
         while (owner_array->a_gp.gp_stub->gs_which == GP_ARRAY)
             owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
-        glist = owner_array->a_gp.gp_stub->gs_un.gs_glist;  
+        glist = owner_array->a_gp.gp_stub->gs_un.gs_glist;
     }
     canvas = (t_pd *)glist_getcanvas(glist);
     if (argc && argv->a_type == A_SYMBOL)
@@ -420,7 +420,7 @@ static void ptrobj_rewind(t_ptrobj *x)
         pd_error(x, "pointer_rewind: sorry, unavailable for arrays");
         return;
     }
-    glist = gs->gs_un.gs_glist;  
+    glist = gs->gs_un.gs_glist;
     gpointer_setglist(&x->x_gp, glist, 0);
     ptrobj_bang(x);
 }
@@ -435,19 +435,19 @@ static void ptrobj_setup(void)
 {
     ptrobj_class = class_new(gensym("pointer"), (t_newmethod)ptrobj_new,
         (t_method)ptrobj_free, sizeof(t_ptrobj), 0, A_GIMME, 0);
-    class_addmethod(ptrobj_class, (t_method)ptrobj_next, gensym("next"), 0); 
-    class_addmethod(ptrobj_class, (t_method)ptrobj_send, gensym("send"), 
-        A_SYMBOL, 0); 
+    class_addmethod(ptrobj_class, (t_method)ptrobj_next, gensym("next"), 0);
+    class_addmethod(ptrobj_class, (t_method)ptrobj_send, gensym("send"),
+        A_SYMBOL, 0);
     class_addmethod(ptrobj_class, (t_method)ptrobj_traverse, gensym("traverse"),
-        A_SYMBOL, 0); 
-    class_addmethod(ptrobj_class, (t_method)ptrobj_vnext, gensym("vnext"), 
-        A_DEFFLOAT, 0); 
+        A_SYMBOL, 0);
+    class_addmethod(ptrobj_class, (t_method)ptrobj_vnext, gensym("vnext"),
+        A_DEFFLOAT, 0);
     class_addmethod(ptrobj_class, (t_method)ptrobj_sendwindow,
-        gensym("send-window"), A_GIMME, 0); 
+        gensym("send-window"), A_GIMME, 0);
     class_addmethod(ptrobj_class, (t_method)ptrobj_rewind,
-        gensym("rewind"), 0); 
-    class_addpointer(ptrobj_class, ptrobj_pointer); 
-    class_addbang(ptrobj_class, ptrobj_bang); 
+        gensym("rewind"), 0);
+    class_addpointer(ptrobj_class, ptrobj_pointer);
+    class_addbang(ptrobj_class, ptrobj_bang);
 }
 
 /* ---------------------- get ----------------------------- */
@@ -503,7 +503,7 @@ static void get_set(t_get *x, t_symbol *templatesym, t_symbol *field)
         pd_error(x, "get: cannot set multiple fields.");
     else
     {
-        x->x_templatesym = template_getbindsym(templatesym); 
+        x->x_templatesym = template_getbindsym(templatesym);
         x->x_variables->gv_sym = field;
     }
 }
@@ -514,7 +514,7 @@ static void get_pointer(t_get *x, t_gpointer *gp)
     t_symbol *templatesym;
     t_template *template;
     t_gstub *gs = gp->gp_stub;
-    t_word *vec; 
+    t_word *vec;
     t_getvariable *vp;
 
     if (!gpointer_check(gp, 0))
@@ -529,7 +529,7 @@ static void get_pointer(t_get *x, t_gpointer *gp)
             pd_error(x, "get %s: got wrong template (%s)",
                 templatesym->s_name, gpointer_gettemplatesym(gp)->s_name);
             return;
-        } 
+        }
     }
     else templatesym = gpointer_gettemplatesym(gp);
     if (!(template = template_findbyname(templatesym)))
@@ -568,9 +568,9 @@ static void get_setup(void)
 {
     get_class = class_new(gensym("get"), (t_newmethod)get_new,
         (t_method)get_free, sizeof(t_get), 0, A_GIMME, 0);
-    class_addpointer(get_class, get_pointer); 
+    class_addpointer(get_class, get_pointer);
     class_addmethod(get_class, (t_method)get_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 }
 
 /* ---------------------- set ----------------------------- */
@@ -642,7 +642,7 @@ static void set_set(t_set *x, t_symbol *templatesym, t_symbol *field)
         pd_error(x, "set: cannot set multiple fields.");
     else
     {
-       x->x_templatesym = template_getbindsym(templatesym); 
+       x->x_templatesym = template_getbindsym(templatesym);
        x->x_variables->gv_sym = field;
        if (x->x_issymbol)
            x->x_variables->gv_w.w_symbol = &s_;
@@ -672,7 +672,7 @@ static void set_bang(t_set *x)
             pd_error(x, "set %s: got wrong template (%s)",
                 templatesym->s_name, gpointer_gettemplatesym(gp)->s_name);
             return;
-        } 
+        }
     }
     else templatesym = gpointer_gettemplatesym(gp);
     if (!(template = template_findbyname(templatesym)))
@@ -691,14 +691,14 @@ static void set_bang(t_set *x)
     else for (i = 0, vp = x->x_variables; i < nitems; i++, vp++)
         template_setfloat(template, vp->gv_sym, vec, vp->gv_w.w_float, 1);
     if (gs->gs_which == GP_GLIST)
-        scalar_redraw(gp->gp_un.gp_scalar, gs->gs_un.gs_glist);  
+        scalar_redraw(gp->gp_un.gp_scalar, gs->gs_un.gs_glist);
     else
     {
         t_array *owner_array = gs->gs_un.gs_array;
         while (owner_array->a_gp.gp_stub->gs_which == GP_ARRAY)
             owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
         scalar_redraw(owner_array->a_gp.gp_un.gp_scalar,
-            owner_array->a_gp.gp_stub->gs_un.gs_glist);  
+            owner_array->a_gp.gp_stub->gs_un.gs_glist);
     }
 }
 
@@ -732,11 +732,11 @@ static void set_setup(void)
 {
     set_class = class_new(gensym("set"), (t_newmethod)set_new,
         (t_method)set_free, sizeof(t_set), 0, A_GIMME, 0);
-    class_addfloat(set_class, set_float); 
-    class_addsymbol(set_class, set_symbol); 
-    class_addbang(set_class, set_bang); 
+    class_addfloat(set_class, set_float);
+    class_addsymbol(set_class, set_symbol);
+    class_addbang(set_class, set_bang);
     class_addmethod(set_class, (t_method)set_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 }
 
 /* ---------------------- elem ----------------------------- */
@@ -780,7 +780,7 @@ static void elem_float(t_elem *x, t_float f)
     t_word *w;
     t_array *array;
     int elemsize, type;
-    
+
     if (!gpointer_check(gparent, 0))
     {
         pd_error(x, "element: empty pointer");
@@ -794,7 +794,7 @@ static void elem_float(t_elem *x, t_float f)
             pd_error(x, "elem %s: got wrong template (%s)",
                 templatesym->s_name, gpointer_gettemplatesym(gparent)->s_name);
             return;
-        } 
+        }
     }
     else templatesym = gpointer_gettemplatesym(gparent);
     if (!(template = template_findbyname(templatesym)))
@@ -835,7 +835,7 @@ static void elem_float(t_elem *x, t_float f)
     if (indx < 0) indx = 0;
     if (indx >= nitems) indx = nitems-1;
 
-    gpointer_setarray(&x->x_gp, array, 
+    gpointer_setarray(&x->x_gp, array,
         (t_word *)((char *)(array->a_vec) + indx * elemsize));
     outlet_pointer(x->x_obj.ob_outlet, &x->x_gp);
 }
@@ -850,9 +850,9 @@ static void elem_setup(void)
 {
     elem_class = class_new(gensym("element"), (t_newmethod)elem_new,
         (t_method)elem_free, sizeof(t_elem), 0, A_DEFSYM, A_DEFSYM, 0);
-    class_addfloat(elem_class, elem_float); 
+    class_addfloat(elem_class, elem_float);
     class_addmethod(elem_class, (t_method)elem_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 }
 
 /* ---------------------- getsize ----------------------------- */
@@ -903,7 +903,7 @@ static void getsize_pointer(t_getsize *x, t_gpointer *gp)
             pd_error(x, "elem %s: got wrong template (%s)",
                 templatesym->s_name, gpointer_gettemplatesym(gp)->s_name);
             return;
-        } 
+        }
     }
     else templatesym = gpointer_gettemplatesym(gp);
     if (!(template = template_findbyname(templatesym)))
@@ -924,7 +924,7 @@ static void getsize_pointer(t_getsize *x, t_gpointer *gp)
     }
     if (gs->gs_which == GP_ARRAY) w = gp->gp_un.gp_w;
     else w = gp->gp_un.gp_scalar->sc_vec;
-    
+
     array = *(t_array **)(((char *)w) + onset);
     outlet_float(x->x_obj.ob_outlet, (t_float)(array->a_n));
 }
@@ -933,9 +933,9 @@ static void getsize_setup(void)
 {
     getsize_class = class_new(gensym("getsize"), (t_newmethod)getsize_new, 0,
         sizeof(t_getsize), 0, A_DEFSYM, A_DEFSYM, 0);
-    class_addpointer(getsize_class, getsize_pointer); 
+    class_addpointer(getsize_class, getsize_pointer);
     class_addmethod(getsize_class, (t_method)getsize_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 }
 
 /* ---------------------- setsize ----------------------------- */
@@ -957,7 +957,7 @@ static void *setsize_new(t_symbol *templatesym, t_symbol *fieldsym,
     x->x_templatesym = template_getbindsym(templatesym);
     x->x_fieldsym = fieldsym;
     gpointer_init(&x->x_gp);
-    
+
     pointerinlet_new(&x->x_obj, &x->x_gp);
     return (x);
 }
@@ -994,7 +994,7 @@ static void setsize_float(t_setsize *x, t_float f)
             pd_error(x, "elem %s: got wrong template (%s)",
                 templatesym->s_name, gpointer_gettemplatesym(gp)->s_name);
             return;
-        } 
+        }
     }
     else templatesym = gpointer_gettemplatesym(gp);
     if (!(template = template_findbyname(templatesym)))
@@ -1033,7 +1033,7 @@ static void setsize_float(t_setsize *x, t_float f)
     nitems = array->a_n;
     if (newsize < 1) newsize = 1;
     if (newsize == nitems) return;
-    
+
         /* erase the array before resizing it.  If we belong to a
         scalar it's easy, but if we belong to an element of another
         array we have to search back until we get to a scalar to erase.
@@ -1043,7 +1043,7 @@ static void setsize_float(t_setsize *x, t_float f)
     if (gs->gs_which == GP_GLIST)
     {
         if (glist_isvisible(gs->gs_un.gs_glist))
-            gobj_vis((t_gobj *)(gp->gp_un.gp_scalar), gs->gs_un.gs_glist, 0);  
+            gobj_vis((t_gobj *)(gp->gp_un.gp_scalar), gs->gs_un.gs_glist, 0);
     }
     else
     {
@@ -1052,13 +1052,13 @@ static void setsize_float(t_setsize *x, t_float f)
             owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
         if (glist_isvisible(owner_array->a_gp.gp_stub->gs_un.gs_glist))
             gobj_vis((t_gobj *)(owner_array->a_gp.gp_un.gp_scalar),
-                owner_array->a_gp.gp_stub->gs_un.gs_glist, 0);  
+                owner_array->a_gp.gp_stub->gs_un.gs_glist, 0);
     }
         /* if shrinking, free the scalars that will disappear */
     if (newsize < nitems)
     {
         char *elem;
-        int count;       
+        int count;
         for (elem = ((char *)array->a_vec) + newsize * elemsize,
             count = nitems - newsize; count--; elem += elemsize)
                 word_free((t_word *)elem, elemtemplate);
@@ -1071,7 +1071,7 @@ static void setsize_float(t_setsize *x, t_float f)
     if (newsize > nitems)
     {
         char *elem;
-        int count;       
+        int count;
         for (elem = ((char *)array->a_vec) + nitems * elemsize,
             count = newsize - nitems; count--; elem += elemsize)
                 word_init((t_word *)elem, elemtemplate, gp);
@@ -1083,7 +1083,7 @@ static void setsize_float(t_setsize *x, t_float f)
     if (gs->gs_which == GP_GLIST)
     {
         if (glist_isvisible(gs->gs_un.gs_glist))
-            gobj_vis((t_gobj *)(gp->gp_un.gp_scalar), gs->gs_un.gs_glist, 1);  
+            gobj_vis((t_gobj *)(gp->gp_un.gp_scalar), gs->gs_un.gs_glist, 1);
     }
     else
     {
@@ -1092,7 +1092,7 @@ static void setsize_float(t_setsize *x, t_float f)
             owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
         if (glist_isvisible(owner_array->a_gp.gp_stub->gs_un.gs_glist))
             gobj_vis((t_gobj *)(owner_array->a_gp.gp_un.gp_scalar),
-                owner_array->a_gp.gp_stub->gs_un.gs_glist, 1);  
+                owner_array->a_gp.gp_stub->gs_un.gs_glist, 1);
     }
 }
 
@@ -1108,7 +1108,7 @@ static void setsize_setup(void)
         A_DEFSYM, A_DEFSYM, A_DEFFLOAT, 0);
     class_addfloat(setsize_class, setsize_float);
     class_addmethod(setsize_class, (t_method)setsize_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 
 }
 
@@ -1167,7 +1167,7 @@ static void append_set(t_append *x, t_symbol *templatesym, t_symbol *field)
         pd_error(x, "set: cannot set multiple fields.");
     else
     {
-       x->x_templatesym = template_getbindsym(templatesym); 
+       x->x_templatesym = template_getbindsym(templatesym);
        x->x_variables->gv_sym = field;
        x->x_variables->gv_f = 0;
     }
@@ -1184,7 +1184,7 @@ static void append_float(t_append *x, t_float f)
     t_word *vec;
     t_scalar *sc, *oldsc;
     t_glist *glist;
-    
+
     if (!templatesym->s_name)
     {
         pd_error(x, "append: no template supplied");
@@ -1222,7 +1222,7 @@ static void append_float(t_append *x, t_float f)
         return;
     }
     oldsc = gp->gp_un.gp_scalar;
-    
+
     if (oldsc)
     {
         sc->sc_gobj.g_next = oldsc->sc_gobj.g_next;
@@ -1240,7 +1240,7 @@ static void append_float(t_append *x, t_float f)
     {
         template_setfloat(template, vp->gv_sym, vec, vp->gv_f, 1);
     }
- 
+
     if (glist_isvisible(glist_getcanvas(glist)))
         gobj_vis(&sc->sc_gobj, glist, 1);
     /*  scalar_redraw(sc, glist);  ... have to do 'vis' instead here because
@@ -1259,9 +1259,9 @@ static void append_setup(void)
 {
     append_class = class_new(gensym("append"), (t_newmethod)append_new,
         (t_method)append_free, sizeof(t_append), 0, A_GIMME, 0);
-    class_addfloat(append_class, append_float); 
+    class_addfloat(append_class, append_float);
     class_addmethod(append_class, (t_method)append_set, gensym("set"),
-        A_SYMBOL, A_SYMBOL, 0); 
+        A_SYMBOL, A_SYMBOL, 0);
 }
 
 /* ----------------- setup function ------------------- */
