@@ -37,8 +37,8 @@ static void vu_update_rms(t_vu *x, t_glist *glist)
         int xpos=text_xpix(&x->x_gui.x_obj, glist), quad1=xpos+w4+1, quad3=xpos+x->x_gui.x_w-w4-1;
 
         sys_vgui(".x%lx.c coords %lxRCOVER %d %d %d %d\n",
-                 glist_getcanvas(glist), x, quad1, off, quad3,
-                 off + (x->x_led_size+1)*(IEM_VU_STEPS-x->x_rms));
+             glist_getcanvas(glist), x, quad1, off, quad3,
+             off + (x->x_led_size+1)*IEMGUI_ZOOM(x)*(IEM_VU_STEPS-x->x_rms));
     }
 }
 
@@ -54,20 +54,20 @@ static void vu_update_peak(t_vu *x, t_glist *glist)
         if(x->x_peak)
         {
             int i=iemgui_vu_col[x->x_peak];
-            int j=ypos + (x->x_led_size+1)*(IEM_VU_STEPS+1-x->x_peak)
-                - (x->x_led_size+1)/2;
+            int j=ypos + (x->x_led_size+1)*IEMGUI_ZOOM(x)*(IEM_VU_STEPS+1-x->x_peak)
+                - (x->x_led_size+1)*IEMGUI_ZOOM(x)/2;
 
             sys_vgui(".x%lx.c coords %lxPLED %d %d %d %d\n", canvas, x,
                      xpos, j,
                      xpos+x->x_gui.x_w+1, j);
-            sys_vgui(".x%lx.c itemconfigure %lxPLED -fill #%6.6x\n", canvas, x,
+            sys_vgui(".x%lx.c itemconfigure %lxPLED -fill #%06x\n", canvas, x,
                      iemgui_color_hex[i]);
         }
         else
         {
             int mid=xpos+x->x_gui.x_w/2;
 
-            sys_vgui(".x%lx.c itemconfigure %lxPLED -fill #%6.6x\n",
+            sys_vgui(".x%lx.c itemconfigure %lxPLED -fill #%06x\n",
                      canvas, x, x->x_gui.x_bcol);
             sys_vgui(".x%lx.c coords %lxPLED %d %d %d %d\n",
                      canvas, x, mid, ypos+20,
@@ -90,7 +90,7 @@ static void vu_draw_update(t_gobj *client, t_glist *glist)
         x->x_updatepeak = 0;
     }
 }
-    
+
 static void vu_draw_new(t_vu *x, t_glist *glist)
 {
     t_canvas *canvas=glist_getcanvas(glist);
@@ -101,23 +101,23 @@ static void vu_draw_new(t_vu *x, t_glist *glist)
         quad1=xpos+w4+1;
     int quad3=xpos+x->x_gui.x_w-w4,
         end=xpos+x->x_gui.x_w+4;
-    int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
+    int k1=(x->x_led_size+1)*IEMGUI_ZOOM(x), k2=IEM_VU_STEPS+1, k3=k1/2;
     int led_col, yyy, i, k4=ypos-k3;
 
-    sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill #%6.6x -tags %lxBASE\n",
+    sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -fill #%06x -tags %lxBASE\n",
              canvas, xpos-1, ypos-2,
              xpos+x->x_gui.x_w+1,
-             ypos+x->x_gui.x_h+2, x->x_gui.x_bcol, x);
+             ypos+x->x_gui.x_h+2, IEMGUI_ZOOM(x), x->x_gui.x_bcol, x);
     for(i=1; i<=IEM_VU_STEPS; i++)
     {
         led_col = iemgui_vu_col[i];
         yyy = k4 + k1*(k2-i);
-        sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%6.6x -tags %lxRLED%d\n",
-                 canvas, quad1, yyy, quad3, yyy, x->x_led_size, iemgui_color_hex[led_col], x, i);
+        sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxRLED%d\n",
+                 canvas, quad1, yyy, quad3, yyy, x->x_led_size*IEMGUI_ZOOM(x), iemgui_color_hex[led_col], x, i);
         if(((i+2)&3) && (x->x_scale))
             sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-                     -font {{%s} -%d %s} -fill #%6.6x -tags %lxSCALE%d\n",
-                     canvas, end, yyy+k3, iemgui_vu_scale_str[i], 
+                     -font {{%s} -%d %s} -fill #%06x -tags %lxSCALE%d\n",
+                     canvas, end, yyy+k3, iemgui_vu_scale_str[i],
                      x->x_gui.x_font, x->x_gui.x_fontsize,
                      sys_fontweight, x->x_gui.x_lcol, x, i);
     }
@@ -126,47 +126,47 @@ static void vu_draw_new(t_vu *x, t_glist *glist)
         i=IEM_VU_STEPS+1;
         yyy = k4 + k1*(k2-i);
         sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-                 -font {{%s} -%d %s} -fill #%6.6x -tags %lxSCALE%d\n",
-                 canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font, 
+                 -font {{%s} -%d %s} -fill #%06x -tags %lxSCALE%d\n",
+                 canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font,
                  x->x_gui.x_fontsize, sys_fontweight,
                  x->x_gui.x_lcol, x, i);
     }
-    sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill #%6.6x -outline #%6.6x -tags %lxRCOVER\n",
+    sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill #%06x -outline #%06x -tags %lxRCOVER\n",
              canvas, quad1, ypos-1, quad3-1,
              ypos-1 + k1*IEM_VU_STEPS, x->x_gui.x_bcol, x->x_gui.x_bcol, x);
-    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%6.6x -tags %lxPLED\n",
+    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxPLED\n",
              canvas, mid, ypos+10,
-             mid, ypos+10, x->x_led_size, x->x_gui.x_bcol, x);
+             mid, ypos+10, (x->x_led_size+1)*IEMGUI_ZOOM(x), x->x_gui.x_bcol, x);
     sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-             -font {{%s} -%d %s} -fill #%6.6x -tags [list %lxLABEL label text]\n",
+             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
              canvas, xpos+x->x_gui.x_ldx, ypos+x->x_gui.x_ldy,
              strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
              x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
              x->x_gui.x_lcol, x);
     if(!x->x_gui.x_fsf.x_snd_able)
     {
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxOUT%d outlet]\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
              canvas,
-             xpos-1, ypos + x->x_gui.x_h+1,
+             xpos-1, ypos + x->x_gui.x_h+3-2*IEMGUI_ZOOM(x),
              xpos + IOWIDTH-1, ypos + x->x_gui.x_h+2,
              x, 0);
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxOUT%d outlet]x\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]x\n",
              canvas,
-             xpos+x->x_gui.x_w+1-IOWIDTH, ypos + x->x_gui.x_h+1,
+             xpos+x->x_gui.x_w+1-IOWIDTH, ypos + x->x_gui.x_h+3-2*IEMGUI_ZOOM(x),
              xpos+x->x_gui.x_w+1, ypos + x->x_gui.x_h+2,
              x, 1);
     }
     if(!x->x_gui.x_fsf.x_rcv_able)
     {
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxIN%d inlet]\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
              canvas,
              xpos-1, ypos-2,
-             xpos + IOWIDTH-1, ypos-1,
+             xpos + IOWIDTH-1, ypos-2+2*IEMGUI_ZOOM(x),
              x, 0);
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxIN%d inlet]\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
              canvas,
              xpos+x->x_gui.x_w+1-IOWIDTH, ypos-2,
-             xpos+x->x_gui.x_w+1, ypos-1,
+             xpos+x->x_gui.x_w+1, ypos-2+2*IEMGUI_ZOOM(x),
              x, 1);
     }
     x->x_updaterms = x->x_updatepeak = 1;
@@ -183,7 +183,7 @@ static void vu_draw_move(t_vu *x, t_glist *glist)
     int w4=x->x_gui.x_w/4, quad1=xpos+w4+1;
     int quad3=xpos+x->x_gui.x_w-w4,
         end=xpos+x->x_gui.x_w+4;
-    int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
+    int k1=(x->x_led_size+1)*IEMGUI_ZOOM(x), k2=IEM_VU_STEPS+1, k3=k1/2;
     int yyy, i, k4=ypos-k3;
 
     sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
@@ -214,11 +214,11 @@ static void vu_draw_move(t_vu *x, t_glist *glist)
     {
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
              canvas, x, 0,
-             xpos-1, ypos + x->x_gui.x_h+1,
+             xpos-1, ypos + x->x_gui.x_h+3-2*IEMGUI_ZOOM(x),
              xpos + IOWIDTH-1, ypos + x->x_gui.x_h+2);
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
              canvas, x, 1,
-             xpos+x->x_gui.x_w+1-IOWIDTH, ypos + x->x_gui.x_h+1,
+             xpos+x->x_gui.x_w+1-IOWIDTH, ypos + x->x_gui.x_h+3-2*IEMGUI_ZOOM(x),
                  xpos+x->x_gui.x_w+1, ypos + x->x_gui.x_h+2);
     }
     if(!x->x_gui.x_fsf.x_rcv_able)
@@ -271,34 +271,34 @@ static void vu_draw_config(t_vu* x, t_glist* glist)
     int i;
     t_canvas *canvas=glist_getcanvas(glist);
 
-    sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
+    sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%06x\n", canvas, x, x->x_gui.x_bcol);
     for(i=1; i<=IEM_VU_STEPS; i++)
     {
         sys_vgui(".x%lx.c itemconfigure %lxRLED%d -width %d\n", canvas, x, i,
-                 x->x_led_size);
+                 (x->x_led_size+1)*IEMGUI_ZOOM(x));
         if(((i+2)&3) && (x->x_scale))
-            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -text {%s} -font {{%s} -%d %s} -fill #%6.6x\n",
-                     canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font, 
-                     x->x_gui.x_fontsize, sys_fontweight, 
+            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -text {%s} -font {{%s} -%d %s} -fill #%06x\n",
+                     canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font,
+                     x->x_gui.x_fontsize, sys_fontweight,
                      x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol);
     }
     if(x->x_scale)
     {
         i=IEM_VU_STEPS+1;
-        sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -text {%s} -font {{%s} -%d %s} -fill #%6.6x\n",
-                 canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font, 
+        sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -text {%s} -font {{%s} -%d %s} -fill #%06x\n",
+                 canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font,
                  x->x_gui.x_fontsize, sys_fontweight,
                  x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol);
     }
-    sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%6.6x -text {%s} \n",
+    sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
              canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
              x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
              strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
 
-    sys_vgui(".x%lx.c itemconfigure %lxRCOVER -fill #%6.6x -outline #%6.6x\n", canvas,
+    sys_vgui(".x%lx.c itemconfigure %lxRCOVER -fill #%06x -outline #%06x\n", canvas,
              x, x->x_gui.x_bcol, x->x_gui.x_bcol);
     sys_vgui(".x%lx.c itemconfigure %lxPLED -width %d\n", canvas, x,
-             x->x_led_size);
+             (x->x_led_size+1)*IEMGUI_ZOOM(x));
 }
 
 static void vu_draw_io(t_vu* x, t_glist* glist, int old_snd_rcv_flags)
@@ -352,37 +352,37 @@ static void vu_draw_select(t_vu* x,t_glist* glist)
 
     if(x->x_gui.x_fsf.x_selected)
     {
-        sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+        sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
         for(i=1; i<=IEM_VU_STEPS; i++)
         {
             if(((i+2)&3) && (x->x_scale))
-                sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%6.6x\n",
+                sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%06x\n",
                          canvas, x, i, IEM_GUI_COLOR_SELECTED);
         }
         if(x->x_scale)
         {
             i=IEM_VU_STEPS+1;
-            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%6.6x\n",
+            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%06x\n",
                      canvas, x, i, IEM_GUI_COLOR_SELECTED);
         }
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
+        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
     }
     else
     {
-        sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
+        sys_vgui(".x%lx.c itemconfigure %lxBASE -outline #%06x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
         for(i=1; i<=IEM_VU_STEPS; i++)
         {
             if(((i+2)&3) && (x->x_scale))
-                sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%6.6x\n",
+                sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%06x\n",
                          canvas, x, i, x->x_gui.x_lcol);
         }
         if(x->x_scale)
         {
             i=IEM_VU_STEPS+1;
-            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%6.6x\n",
+            sys_vgui(".x%lx.c itemconfigure %lxSCALE%d -fill #%06x\n",
                      canvas, x, i, x->x_gui.x_lcol);
         }
-        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
+        sys_vgui(".x%lx.c itemconfigure %lxLABEL -fill #%06x\n", canvas, x, x->x_gui.x_lcol);
     }
 }
 
@@ -419,11 +419,11 @@ static void vu_getrect(t_gobj *z, t_glist *glist,
 static void vu_save(t_gobj *z, t_binbuf *b)
 {
     t_vu *x = (t_vu *)z;
-    int bflcol[3];
+    t_symbol *bflcol[3];
     t_symbol *srl[3];
 
     iemgui_save(&x->x_gui, srl, bflcol);
-    binbuf_addv(b, "ssiisiissiiiiiiii", gensym("#X"),gensym("obj"),
+    binbuf_addv(b, "ssiisiissiiiissii", gensym("#X"),gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix, (int)x->x_gui.x_obj.te_ypix,
                 gensym("vu"), x->x_gui.x_w, x->x_gui.x_h,
                 srl[1], srl[2],
@@ -469,7 +469,7 @@ static void vu_scale(t_vu *x, t_floatarg fscale)
     if(!x->x_scale && scale)
     {
         int w4=x->x_gui.x_w/4, end=text_xpix(&x->x_gui.x_obj, x->x_gui.x_glist)+x->x_gui.x_w+4;
-        int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
+        int k1=(x->x_led_size+1)*IEMGUI_ZOOM(x), k2=IEM_VU_STEPS+1, k3=k1/2;
         int yyy, k4=text_ypix(&x->x_gui.x_obj, x->x_gui.x_glist)-k3;
         t_canvas *canvas=glist_getcanvas(x->x_gui.x_glist);
 
@@ -481,16 +481,16 @@ static void vu_scale(t_vu *x, t_floatarg fscale)
                 yyy = k4 + k1*(k2-i);
                 if((i+2)&3)
                     sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-                             -font {{%s} -%d %s} -fill #%6.6x -tags %lxSCALE%d\n",
-                             canvas, end, yyy+k3, iemgui_vu_scale_str[i], 
+                             -font {{%s} -%d %s} -fill #%06x -tags %lxSCALE%d\n",
+                             canvas, end, yyy+k3, iemgui_vu_scale_str[i],
                              x->x_gui.x_font, x->x_gui.x_fontsize,
                              sys_fontweight, x->x_gui.x_lcol, x, i);
             }
             i=IEM_VU_STEPS+1;
             yyy = k4 + k1*(k2-i);
             sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-                     -font {{%s} -%d %s} -fill #%6.6x -tags %lxSCALE%d\n",
-                     canvas, end, yyy+k3, iemgui_vu_scale_str[i], 
+                     -font {{%s} -%d %s} -fill #%06x -tags %lxSCALE%d\n",
+                     canvas, end, yyy+k3, iemgui_vu_scale_str[i],
                      x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
                      sys_fontweight, x->x_gui.x_lcol, x, i);
         }
@@ -511,14 +511,14 @@ static void vu_properties(t_gobj *z, t_glist *owner)
             %s %s \
             %s %d %d \
             %d %d \
-            %d %d %d\n",
+            #%06x none #%06x\n",
             x->x_gui.x_w, IEM_GUI_MINSIZE, x->x_gui.x_h, IEM_VU_STEPS*IEM_VU_MINSIZE,
             0,/*no_schedule*/
             x->x_scale, -1, -1, -1,/*no linlog, no init, no multi*/
             "nosndno", srl[1]->s_name,/*no send*/
             srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
             x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
-            0xffffff & x->x_gui.x_bcol, -1/*no front-color*/, 0xffffff & x->x_gui.x_lcol);
+            0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_lcol);
     gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
 }
 
@@ -634,7 +634,6 @@ static void vu_bang(t_vu *x)
 static void *vu_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_vu *x = (t_vu *)pd_new(vu_class);
-    int bflcol[]={-66577, -1, -1};
     int w=IEM_GUI_DEFAULTSIZE, h=IEM_VU_STEPS*IEM_VU_DEFAULTSIZE;
     int ldx=-1, ldy=-8, f=0, fs=10, scale=1;
     int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME, fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
@@ -643,12 +642,16 @@ static void *vu_new(t_symbol *s, int argc, t_atom *argv)
     iem_inttosymargs(&x->x_gui.x_isa, 0);
     iem_inttofstyle(&x->x_gui.x_fsf, 0);
 
+    x->x_gui.x_bcol = 0x404040;
+    x->x_gui.x_fcol = 0x00;
+    x->x_gui.x_lcol = 0x00;
+
     if((argc >= 11)&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
        &&(IS_A_SYMBOL(argv,2)||IS_A_FLOAT(argv,2))
        &&(IS_A_SYMBOL(argv,3)||IS_A_FLOAT(argv,3))
        &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
        &&IS_A_FLOAT(argv,6)&&IS_A_FLOAT(argv,7)
-       &&IS_A_FLOAT(argv,8)&&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10))
+       &&IS_A_FLOAT(argv,10))
     {
         w = (int)atom_getintarg(0, argc, argv);
         h = (int)atom_getintarg(1, argc, argv);
@@ -657,8 +660,7 @@ static void *vu_new(t_symbol *s, int argc, t_atom *argv)
         ldy = (int)atom_getintarg(5, argc, argv);
         iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(6, argc, argv));
         fs = (int)atom_getintarg(7, argc, argv);
-        bflcol[0] = (int)atom_getintarg(8, argc, argv);
-        bflcol[2] = (int)atom_getintarg(9, argc, argv);
+        iemgui_all_loadcolors(&x->x_gui, argv+8, NULL, argv+9);
         scale = (int)atom_getintarg(10, argc, argv);
     }
     else iemgui_new_getnames(&x->x_gui, 1, 0);
@@ -681,13 +683,11 @@ static void *vu_new(t_symbol *s, int argc, t_atom *argv)
         pd_bind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     x->x_gui.x_ldx = ldx;
     x->x_gui.x_ldy = ldy;
-
     if(fs < 4)
         fs = 4;
     x->x_gui.x_fontsize = fs;
     x->x_gui.x_w = iemgui_clip_size(w);
     vu_check_height(x, h);
-    iemgui_all_colfromload(&x->x_gui, bflcol);
     if(scale != 0)
         scale = 1;
     x->x_scale = scale;
@@ -727,6 +727,8 @@ void g_vumeter_setup(void)
     class_addmethod(vu_class, (t_method)vu_label, gensym("label"), A_DEFSYM, 0);
     class_addmethod(vu_class, (t_method)vu_label_pos, gensym("label_pos"), A_GIMME, 0);
     class_addmethod(vu_class, (t_method)vu_label_font, gensym("label_font"), A_GIMME, 0);
+    class_addmethod(vu_class, (t_method)iemgui_zoom, gensym("zoom"),
+        A_CANT, 0);
     vu_widgetbehavior.w_getrectfn =    vu_getrect;
     vu_widgetbehavior.w_displacefn =   iemgui_displace;
     vu_widgetbehavior.w_selectfn =     iemgui_select;
