@@ -19,13 +19,13 @@ moment it also defines "text" but it may later be better to split this off. */
 extern t_pd *newest;    /* OK - this should go into a .h file now :) */
 static t_class *text_define_class;
 
-#ifdef HAVE_ALLOCA_H        /* ifdef nonsense to find include for alloca() */
-# include <alloca.h>        /* linux, mac, mingw, cygwin */
-#elif defined _MSC_VER
-# include <malloc.h>        /* MSVC */
+#ifdef _WIN32
+# include <malloc.h> /* MSVC or mingw on windows */
+#elif defined(__linux__) || defined(__APPLE__)
+# include <alloca.h> /* linux, mac, mingw, cygwin */
 #else
-# include <stddef.h>        /* BSDs for example */
-#endif                      /* end alloca() ifdef nonsense */
+# include <stdlib.h> /* BSDs for example */
+#endif
 
 #ifndef HAVE_ALLOCA     /* can work without alloca() but we never need it */
 #define HAVE_ALLOCA 1
@@ -89,8 +89,9 @@ static void textbuf_open(t_textbuf *x)
     {
         char buf[40];
         sys_vgui("pdtk_textwindow_open .x%lx %dx%d {%s: %s} %d\n",
-            x, 600, 340, "myname", "text", 
-                 sys_hostfontsize(glist_getfont(x->b_canvas)));
+            x, 600, 340, "myname", "text",
+                 sys_hostfontsize(glist_getfont(x->b_canvas),
+                    glist_getzoom(x->b_canvas)));
         sprintf(buf, ".x%lx", (unsigned long)x);
         x->b_guiconnect = guiconnect_new(&x->b_ob.ob_pd, gensym(buf));
         textbuf_senditup(x);
@@ -104,7 +105,7 @@ static void textbuf_close(t_textbuf *x)
     {
         guiconnect_notarget(x->b_guiconnect, 1000);
         x->b_guiconnect = 0;
-    }    
+    }
 }
 
 static void textbuf_addline(t_textbuf *b, t_symbol *s, int argc, t_atom *argv)
@@ -200,7 +201,7 @@ static void textbuf_free(t_textbuf *x)
         guiconnect_notarget(x->b_guiconnect, 1000);
     }
         /* just in case we're still bound to #A from loading... */
-    while (x2 = pd_findbyclass(gensym("#A"), text_define_class))
+    while ((x2 = pd_findbyclass(gensym("#A"), text_define_class)))
         pd_unbind(x2, gensym("#A"));
 }
 
@@ -286,7 +287,7 @@ static void *text_define_new(t_symbol *s, int argc, t_atom *argv)
     asym->s_thing = 0;
         /* and now bind #A to us to receive following messages in the
         saved file or copy buffer */
-    pd_bind(&x->x_ob.ob_pd, asym); 
+    pd_bind(&x->x_ob.ob_pd, asym);
     return (x);
 }
 
@@ -312,7 +313,7 @@ static void text_define_frompointer(t_text_define *x, t_gpointer *gp,
         t_gstub *gs = gp->gp_stub;
         binbuf_clear(x->x_textbuf.b_binbuf);
         binbuf_add(x->x_textbuf.b_binbuf, binbuf_getnatom(b), binbuf_getvec(b));
-    } 
+    }
 }
 
 static void text_define_topointer(t_text_define *x, t_gpointer *gp, t_symbol *s)
@@ -326,16 +327,16 @@ static void text_define_topointer(t_text_define *x, t_gpointer *gp, t_symbol *s)
         binbuf_add(b, binbuf_getnatom(x->x_textbuf.b_binbuf),
             binbuf_getvec(x->x_textbuf.b_binbuf));
         if (gs->gs_which == GP_GLIST)
-            scalar_redraw(gp->gp_un.gp_scalar, gs->gs_un.gs_glist);  
+            scalar_redraw(gp->gp_un.gp_scalar, gs->gs_un.gs_glist);
         else
         {
             t_array *owner_array = gs->gs_un.gs_array;
             while (owner_array->a_gp.gp_stub->gs_which == GP_ARRAY)
                 owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
             scalar_redraw(owner_array->a_gp.gp_un.gp_scalar,
-                owner_array->a_gp.gp_stub->gs_un.gs_glist);  
+                owner_array->a_gp.gp_stub->gs_un.gs_glist);
         }
-    } 
+    }
 }
 
     /* bang: output a pointer to a struct containing this text */
@@ -445,7 +446,7 @@ static t_binbuf *text_client_getbuf(t_text_client *x)
     {
         t_template *template = template_findbyname(x->tc_struct);
         t_gstub *gs = x->tc_gp.gp_stub;
-        t_word *vec; 
+        t_word *vec;
         int onset, type;
         t_symbol *arraytype;
         if (!template)
@@ -503,14 +504,14 @@ static  void text_client_senditup(t_text_client *x)
             return;
         }
         if (gs->gs_which == GP_GLIST)
-            scalar_redraw(x->tc_gp.gp_un.gp_scalar, gs->gs_un.gs_glist);  
+            scalar_redraw(x->tc_gp.gp_un.gp_scalar, gs->gs_un.gs_glist);
         else
         {
             t_array *owner_array = gs->gs_un.gs_array;
             while (owner_array->a_gp.gp_stub->gs_which == GP_ARRAY)
                 owner_array = owner_array->a_gp.gp_stub->gs_un.gs_array;
             scalar_redraw(owner_array->a_gp.gp_un.gp_scalar,
-                owner_array->a_gp.gp_stub->gs_un.gs_glist);  
+                owner_array->a_gp.gp_stub->gs_un.gs_glist);
         }
     }
 }
@@ -608,7 +609,7 @@ static void text_get_float(t_text_get *x, t_floatarg f)
         }
         else if (startfield + nfield > outc)
             pd_error(x, "text get: field request (%d %d) out of range",
-                startfield, nfield); 
+                startfield, nfield);
         else
         {
             ATOMS_ALLOCA(outv, nfield);
@@ -642,14 +643,14 @@ static void *text_set_new(t_symbol *s, int argc, t_atom *argv)
     floatinlet_new(&x->x_obj, &x->x_f2);
     x->x_f1 = 0;
     x->x_f2 = -1;
-    text_client_argparse(&x->x_tc, &argc, &argv, "text get");
+    text_client_argparse(&x->x_tc, &argc, &argv, "text set");
     if (argc)
     {
         if (argv->a_type == A_FLOAT)
             x->x_f1 = argv->a_w.w_float;
         else
         {
-            post("text get: can't understand field number");
+            post("text set: can't understand field number");
             postatom(argc, argv); endpost();
         }
         argc--; argv++;
@@ -660,7 +661,7 @@ static void *text_set_new(t_symbol *s, int argc, t_atom *argv)
             x->x_f2 = argv->a_w.w_float;
         else
         {
-            post("text get: can't understand field count");
+            post("text set: can't understand field count");
             postatom(argc, argv); endpost();
         }
         argc--; argv++;
@@ -705,7 +706,10 @@ static void text_set_list(t_text_set *x,
                 memmove(&vec[start + argc], &vec[end],
                     sizeof(*vec) * (oldn - end));
                 if (n < oldn)
+                {
                     (void)binbuf_resize(b, n);
+                    vec = binbuf_getvec(b);
+                }
             }
         }
         else
@@ -746,6 +750,55 @@ static void text_set_list(t_text_set *x,
     text_client_senditup(&x->x_tc);
 }
 
+/* --------- text_delete object - delete nth line ----------- */
+typedef struct _text_delete
+{
+    t_text_client x_tc;
+} t_text_delete;
+
+t_class *text_delete_class;
+
+static void *text_delete_new(t_symbol *s, int argc, t_atom *argv)
+{
+    t_text_delete *x = (t_text_delete *)pd_new(text_delete_class);
+    text_client_argparse(&x->x_tc, &argc, &argv, "text delete");
+    if (argc)
+    {
+        post("warning: text delete ignoring extra argument: ");
+        postatom(argc, argv); endpost();
+    }
+    if (x->x_struct)
+        pointerinlet_new(&x->x_obj, &x->x_gp);
+    else symbolinlet_new(&x->x_obj, &x->x_tc.tc_sym);
+    return (x);
+}
+
+static void text_delete_float(t_text_delete *x, t_floatarg f)
+{
+    t_binbuf *b = text_client_getbuf(&x->x_tc);
+    int start, end, n, lineno = f;
+    t_atom *vec;
+    if (!b)
+       return;
+    vec = binbuf_getvec(b);
+    n = binbuf_getnatom(b);
+    if (lineno < 0)
+        binbuf_clear(b);
+    else if (text_nthline(n, vec, lineno, &start, &end))
+    {
+        if (end < n)
+            end++;
+        memmove(&vec[start], &vec[end], sizeof(*vec) * (n - end));
+        (void)binbuf_resize(b, n - (end - start));
+    }
+    else
+    {
+        post("text delete: %d: line number out of range", lineno);
+        return;
+    }
+    text_client_senditup(&x->x_tc);
+}
+
 /* ---------------- text_size object - output number of lines -------------- */
 t_class *text_size_class;
 
@@ -891,7 +944,6 @@ typedef struct _text_search
 {
     t_text_client x_tc;
     t_outlet *x_out1;       /* line indices */
-    t_outlet *x_out2;       /* number of matches */
     int x_nkeys;
     t_key *x_keyvec;
 } t_text_search;
@@ -901,7 +953,6 @@ static void *text_search_new(t_symbol *s, int argc, t_atom *argv)
     t_text_search *x = (t_text_search *)pd_new(text_search_class);
     int i, key, nkey, nextop;
     x->x_out1 = outlet_new(&x->x_obj, &s_list);
-    x->x_out2 = outlet_new(&x->x_obj, &s_bang);
     text_client_argparse(&x->x_tc, &argc, &argv, "text search");
     for (i = nkey = 0; i < argc; i++)
         if (argv[i].a_type == A_FLOAT)
@@ -911,7 +962,7 @@ static void *text_search_new(t_symbol *s, int argc, t_atom *argv)
     x->x_nkeys = nkey;
     x->x_keyvec = (t_key *)getbytes(nkey * sizeof(*x->x_keyvec));
     if (!argc)
-        x->x_keyvec[0].k_field = 0, x->x_keyvec[0].k_binop = KB_EQ; 
+        x->x_keyvec[0].k_field = 0, x->x_keyvec[0].k_binop = KB_EQ;
     else for (i = key = 0, nextop = -1; i < argc; i++)
     {
         if (argv[i].a_type == A_FLOAT)
@@ -952,7 +1003,7 @@ static void text_search_list(t_text_search *x,
     t_symbol *s, int argc, t_atom *argv)
 {
     t_binbuf *b = text_client_getbuf(&x->x_tc);
-    int i, j, n, lineno, bestline = -1, beststart, bestn, thisstart, thisn,
+    int i, j, n, lineno, bestline = -1, beststart=-1, bestn, thisstart, thisn,
         nkeys = x->x_nkeys, failed = 0;
     t_atom *vec;
     t_key *kp = x->x_keyvec;
@@ -997,6 +1048,7 @@ static void text_search_list(t_text_search *x,
                             if (vec[thisstart+field].a_w.w_float <
                                 argv[j].a_w.w_float)
                                     goto nomatch;
+                        break;
                         case KB_LT:
                             if (vec[thisstart+field].a_w.w_float >=
                                 argv[j].a_w.w_float)
@@ -1045,8 +1097,9 @@ static void text_search_list(t_text_search *x,
                             bug("text search 2");
                     if (argv[j].a_type == A_FLOAT)      /* arg is a float */
                     {
-                        float thisv = vec[thisstart+field].a_w.w_float, 
-                            bestv = vec[beststart+field].a_w.w_float;
+                        float thisv = vec[thisstart+field].a_w.w_float,
+                            bestv = (beststart >= 0 ?
+                                vec[beststart+field].a_w.w_float : -1e20);
                         switch (binop)
                         {
                             case KB_GT:
@@ -1062,6 +1115,7 @@ static void text_search_list(t_text_search *x,
                                     goto replace;
                                 else if (thisv < bestv)
                                     goto nomatch;
+                            break;
                             case KB_NEAR:
                                 if (thisv >= argv[j].a_w.w_float &&
                                     bestv >= argv[j].a_w.w_float)
@@ -1087,12 +1141,12 @@ static void text_search_list(t_text_search *x,
                                         d1 = -d1;
                                     if (d2 < 0)
                                         d2 = -d2;
-                                        
+
                                     if (d1 < d2)
                                         goto replace;
                                     else if (d1 > d2)
                                         goto nomatch;
-                                }   
+                                }
                             break;
                                 /* the other possibility ('=') never decides */
                         }
@@ -1206,7 +1260,7 @@ static void *text_sequence_new(t_symbol *s, int argc, t_atom *argv)
     if (global)
     {
         if (x->x_waitargc)
-            pd_error(x, 
+            pd_error(x,
        "warning: text sequence: numeric 'w' argument ignored if '-g' given");
         x->x_waitargc = 0x40000000;
     }
@@ -1235,9 +1289,9 @@ static void text_sequence_doit(t_text_sequence *x, int argc, t_atom *argv)
         /* test if leading numbers, or a leading symbol equal to our
         "wait symbol", are directing us to wait */
     if (!x->x_lastto && (
-        vec[onset].a_type == A_FLOAT && x->x_waitargc && !x->x_eaten ||
-            vec[onset].a_type == A_SYMBOL &&
-                vec[onset].a_w.w_symbol == x->x_waitsym))
+        (vec[onset].a_type == A_FLOAT && x->x_waitargc && !x->x_eaten) ||
+            (vec[onset].a_type == A_SYMBOL &&
+                vec[onset].a_w.w_symbol == x->x_waitsym)))
     {
         if (vec[onset].a_type == A_FLOAT)
         {
@@ -1392,9 +1446,9 @@ static void text_sequence_tick(t_text_sequence *x)  /* clock callback */
     while (x->x_auto)
     {
         x->x_loop = 1;
-        while (x->x_loop)  
+        while (x->x_loop)
             text_sequence_doit(x, x->x_argc, x->x_argv);
-        if (x->x_nextdelay > 0) 
+        if (x->x_nextdelay > 0)
             break;
     }
     if (x->x_auto)
@@ -1476,6 +1530,8 @@ static void *text_new(t_symbol *s, int argc, t_atom *argv)
             newest = text_get_new(s, argc-1, argv+1);
         else if (!strcmp(str, "set"))
             newest = text_set_new(s, argc-1, argv+1);
+        else if (!strcmp(str, "delete"))
+            newest = text_delete_new(s, argc-1, argv+1);
         else if (!strcmp(str, "size"))
             newest = text_size_new(s, argc-1, argv+1);
         else if (!strcmp(str, "tolist"))
@@ -1486,7 +1542,7 @@ static void *text_new(t_symbol *s, int argc, t_atom *argv)
             newest = text_search_new(s, argc-1, argv+1);
         else if (!strcmp(str, "sequence"))
             newest = text_sequence_new(s, argc-1, argv+1);
-        else 
+        else
         {
             error("list %s: unknown function", str);
             newest = 0;
@@ -1498,7 +1554,7 @@ static void *text_new(t_symbol *s, int argc, t_atom *argv)
 /*  the qlist and textfile objects, as of 0.44, are 'derived' from
 * the text object above.  Maybe later it will be desirable to add new
 * functionality to textfile; qlist is an ancient holdover (1987) and
-* is probably best left alone. 
+* is probably best left alone.
 */
 
 typedef struct _qlist
@@ -1603,7 +1659,7 @@ static void qlist_donext(t_qlist *x, int drop, int automatic)
             ap++;
             onset++;
             count--;
-            if (!count) 
+            if (!count)
             {
                 x->x_onset = onset2;
                 continue;
@@ -1612,7 +1668,7 @@ static void qlist_donext(t_qlist *x, int drop, int automatic)
         wasrewound = x->x_rewound;
         x->x_rewound = 0;
         if (!drop)
-        {   
+        {
             if (ap->a_type == A_FLOAT)
                 typedmess(target, &s_list, count, ap);
             else if (ap->a_type == A_SYMBOL)
@@ -1817,14 +1873,14 @@ static void text_template_init( void)
     if (text_templatecanvas)
         return;
     b = binbuf_new();
-    
+
     glob_setfilename(0, gensym("_text_template"), gensym("."));
     binbuf_text(b, text_templatefile, strlen(text_templatefile));
     binbuf_eval(b, &pd_canvasmaker, 0, 0);
     vmess(s__X.s_thing, gensym("pop"), "i", 0);
-    
+
     glob_setfilename(0, &s_, &s_);
-    binbuf_free(b);  
+    binbuf_free(b);
 }
 
 void x_qlist_setup(void )
@@ -1837,7 +1893,7 @@ void x_qlist_setup(void )
         gensym("click"), 0);
     class_addmethod(text_define_class, (t_method)textbuf_close,
         gensym("close"), 0);
-    class_addmethod(text_define_class, (t_method)textbuf_addline, 
+    class_addmethod(text_define_class, (t_method)textbuf_addline,
         gensym("addline"), A_GIMME, 0);
     class_addmethod(text_define_class, (t_method)text_define_set,
         gensym("set"), A_GIMME, 0);
@@ -1858,13 +1914,19 @@ void x_qlist_setup(void )
             sizeof(t_text_get), 0, A_GIMME, 0);
     class_addfloat(text_get_class, text_get_float);
     class_sethelpsymbol(text_get_class, gensym("text-object"));
-    
+
     text_set_class = class_new(gensym("text set"),
         (t_newmethod)text_set_new, (t_method)text_client_free,
             sizeof(t_text_set), 0, A_GIMME, 0);
     class_addlist(text_set_class, text_set_list);
     class_sethelpsymbol(text_set_class, gensym("text-object"));
-    
+
+    text_delete_class = class_new(gensym("text delete"),
+        (t_newmethod)text_delete_new, (t_method)text_client_free,
+            sizeof(t_text_delete), 0, A_GIMME, 0);
+    class_addfloat(text_delete_class, text_delete_float);
+    class_sethelpsymbol(text_delete_class, gensym("text-object"));
+
     text_size_class = class_new(gensym("text size"),
         (t_newmethod)text_size_new, (t_method)text_client_free,
             sizeof(t_text_size), 0, A_GIMME, 0);
@@ -1883,7 +1945,7 @@ void x_qlist_setup(void )
             sizeof(t_text_fromlist), 0, A_GIMME, 0);
     class_addlist(text_fromlist_class, text_fromlist_list);
     class_sethelpsymbol(text_fromlist_class, gensym("text-object"));
- 
+
     text_search_class = class_new(gensym("text search"),
         (t_newmethod)text_search_new, (t_method)text_client_free,
             sizeof(t_text_search), 0, A_GIMME, 0);
@@ -1893,17 +1955,17 @@ void x_qlist_setup(void )
     text_sequence_class = class_new(gensym("text sequence"),
         (t_newmethod)text_sequence_new, (t_method)text_sequence_free,
             sizeof(t_text_sequence), 0, A_GIMME, 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_step, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_step,
         gensym("step"), 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_line, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_line,
         gensym("line"), A_FLOAT, 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_auto, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_auto,
         gensym("auto"), 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_stop, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_stop,
         gensym("stop"), 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_args, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_args,
         gensym("args"), A_GIMME, 0);
-    class_addmethod(text_sequence_class, (t_method)text_sequence_tempo, 
+    class_addmethod(text_sequence_class, (t_method)text_sequence_tempo,
         gensym("tempo"), A_FLOAT, A_SYMBOL, 0);
     class_addlist(text_sequence_class, text_sequence_list);
     class_sethelpsymbol(text_sequence_class, gensym("text-object"));
@@ -1912,7 +1974,7 @@ void x_qlist_setup(void )
         (t_method)qlist_free, sizeof(t_qlist), 0, 0);
     class_addmethod(qlist_class, (t_method)qlist_rewind, gensym("rewind"), 0);
     class_addmethod(qlist_class, (t_method)qlist_next,
-        gensym("next"), A_DEFFLOAT, 0);  
+        gensym("next"), A_DEFFLOAT, 0);
     class_addmethod(qlist_class, (t_method)qlist_set, gensym("set"),
         A_GIMME, 0);
     class_addmethod(qlist_class, (t_method)qlist_clear, gensym("clear"), 0);
@@ -1928,7 +1990,7 @@ void x_qlist_setup(void )
         A_SYMBOL, A_DEFSYM, 0);
     class_addmethod(qlist_class, (t_method)textbuf_open, gensym("click"), 0);
     class_addmethod(qlist_class, (t_method)textbuf_close, gensym("close"), 0);
-    class_addmethod(qlist_class, (t_method)textbuf_addline, 
+    class_addmethod(qlist_class, (t_method)textbuf_addline,
         gensym("addline"), A_GIMME, 0);
     class_addmethod(qlist_class, (t_method)qlist_print, gensym("print"),
         A_DEFSYM, 0);
@@ -1949,14 +2011,14 @@ void x_qlist_setup(void )
         A_GIMME, 0);
     class_addmethod(textfile_class, (t_method)qlist_add, gensym("append"),
         A_GIMME, 0);
-    class_addmethod(textfile_class, (t_method)qlist_read, gensym("read"), 
+    class_addmethod(textfile_class, (t_method)qlist_read, gensym("read"),
         A_SYMBOL, A_DEFSYM, 0);
-    class_addmethod(textfile_class, (t_method)qlist_write, gensym("write"), 
+    class_addmethod(textfile_class, (t_method)qlist_write, gensym("write"),
         A_SYMBOL, A_DEFSYM, 0);
     class_addmethod(textfile_class, (t_method)textbuf_open, gensym("click"), 0);
-    class_addmethod(textfile_class, (t_method)textbuf_close, gensym("close"), 
+    class_addmethod(textfile_class, (t_method)textbuf_close, gensym("close"),
         0);
-    class_addmethod(textfile_class, (t_method)textbuf_addline, 
+    class_addmethod(textfile_class, (t_method)textbuf_addline,
         gensym("addline"), A_GIMME, 0);
     class_addmethod(textfile_class, (t_method)qlist_print, gensym("print"),
         A_DEFSYM, 0);
