@@ -59,6 +59,8 @@ proc ::pd_menus::configure_for_pdwindow {} {
     # Edit menu
     $menubar.edit entryconfigure [_ "Duplicate"] -state disabled
     $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
+    $menubar.edit entryconfigure [_ "Zoom In"] -state disabled
+    $menubar.edit entryconfigure [_ "Zoom Out"] -state disabled
     $menubar.edit entryconfigure [_ "Edit Mode"] -state disabled
     pdtk_canvas_editmode .pdwindow 0
     # Undo/Redo change names, they need to have the asterisk (*) after
@@ -81,6 +83,8 @@ proc ::pd_menus::configure_for_canvas {mytoplevel} {
     # Edit menu
     $menubar.edit entryconfigure [_ "Duplicate"] -state normal
     $menubar.edit entryconfigure [_ "Tidy Up"] -state normal
+    $menubar.edit entryconfigure [_ "Zoom In"] -state normal
+    $menubar.edit entryconfigure [_ "Zoom Out"] -state normal
     $menubar.edit entryconfigure [_ "Edit Mode"] -state normal
     pdtk_canvas_editmode $mytoplevel $::editmode($mytoplevel)
     # Put menu
@@ -107,6 +111,8 @@ proc ::pd_menus::configure_for_dialog {mytoplevel} {
     # Edit menu
     $menubar.edit entryconfigure [_ "Duplicate"] -state disabled
     $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
+    $menubar.edit entryconfigure [_ "Zoom In"] -state disabled
+    $menubar.edit entryconfigure [_ "Zoom Out"] -state disabled
     $menubar.edit entryconfigure [_ "Edit Mode"] -state disabled
     pdtk_canvas_editmode $mytoplevel 0
     # Undo/Redo change names, they need to have the asterisk (*) after
@@ -168,6 +174,10 @@ proc ::pd_menus::build_edit_menu {mymenu} {
         $mymenu add command -label [_ "Font"] \
             -command {menu_font_dialog}
     }
+    $mymenu add command -label [_ "Zoom In"] -accelerator "$accelerator++" \
+        -command {menu_send_float $::focused_window zoom 2}
+    $mymenu add command -label [_ "Zoom Out"] -accelerator "$accelerator+-" \
+        -command {menu_send_float $::focused_window zoom 1}
     $mymenu add command -label [_ "Tidy Up"] \
         -command {menu_send $::focused_window tidy}
     $mymenu add command -label [_ "Clear Console"] \
@@ -264,6 +274,7 @@ proc ::pd_menus::build_media_menu {mymenu} {
             -command {pdsend "pd midi-setapi $::pd_whichmidiapi"}
     }
 
+    $mymenu add  separator
     $mymenu add command -label [_ "Audio Settings..."] \
         -command {pdsend "pd audio-properties"}
     $mymenu add command -label [_ "MIDI Settings..."] \
@@ -304,15 +315,13 @@ proc ::pd_menus::build_help_menu {mymenu} {
         -command {menu_doc_open doc/1.manual index.htm}
     $mymenu add command -label [_ "Browser..."] \
         -command {menu_helpbrowser} 
+    $mymenu add command -label [_ "List of objects..."] \
+        -command {pdsend "pd help-intro"} 
     $mymenu add  separator
     $mymenu add command -label [_ "puredata.info"] \
         -command {menu_openfile {http://puredata.info}} 
     $mymenu add command -label [_ "Report a bug"] -command {menu_openfile \
         {http://sourceforge.net/tracker/?func=add&group_id=55736&atid=478070}} 
-    $mymenu add  separator
-    $mymenu add command -label [_ "Tcl prompt"] -command \
-        {::pdwindow::create_tcl_entry} 
-
 }
 
 #------------------------------------------------------------------------------#
@@ -484,6 +493,9 @@ proc ::pd_menus::create_preferences_menu {mymenu} {
         -command {pdsend "pd audio-properties"}
     $mymenu add command -label [_ "MIDI Settings..."] \
         -command {pdsend "pd midi-properties"}
+    $mymenu add check -label [_ "Zoom new windows"] \
+        -variable ::zoom_open \
+        -command {pdsend "pd zoom-open $zoom_open"}
     $mymenu add  separator
     $mymenu add command -label [_ "Save All Settings"] \
         -command {pdsend "pd save-preferences"}
